@@ -7,7 +7,7 @@ use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList, PyString};
 use pyo3::{intern, FromPyObject, PyErrArguments};
 
-use crate::errors::ValError;
+use crate::errors::{ErrorMode, ValError};
 use crate::ValidationError;
 
 pub trait SchemaDict<'py> {
@@ -99,6 +99,18 @@ pub fn is_strict(schema: &PyDict, config: Option<&PyDict>) -> PyResult<bool> {
     Ok(schema_or_config_same(schema, config, intern!(py, "strict"))?.unwrap_or(false))
 }
 
+pub fn destructure_function_schema(schema: &PyDict) -> PyResult<(bool, &PyAny)> {
+    let func_dict: &PyDict = schema.get_as_req(intern!(schema.py(), "function"))?;
+    let function: &PyAny = func_dict.get_as_req(intern!(schema.py(), "function"))?;
+    let func_type: &str = func_dict.get_as_req(intern!(schema.py(), "type"))?;
+    let is_field_serializer = match func_type {
+        "field" => true,
+        "general" => false,
+        _ => unreachable!(),
+    };
+    Ok((is_field_serializer, function))
+}
+
 enum SchemaErrorEnum {
     Message(String),
     ValidationError(ValidationError),
@@ -140,7 +152,7 @@ impl SchemaError {
         match error {
             ValError::LineErrors(raw_errors) => {
                 let line_errors = raw_errors.into_iter().map(|e| e.into_py(py)).collect();
-                let validation_error = ValidationError::new(line_errors, "Schema".to_object(py));
+                let validation_error = ValidationError::new(line_errors, "Schema".to_object(py), ErrorMode::Python);
                 let schema_error = SchemaError(SchemaErrorEnum::ValidationError(validation_error));
                 match Py::new(py, schema_error) {
                     Ok(err) => PyErr::from_value(err.into_ref(py)),
@@ -248,3 +260,35 @@ pub fn safe_repr(v: &PyAny) -> Cow<str> {
         "<unprintable object>".into()
     }
 }
+
+#[derive(Debug, Clone)]
+pub(crate) enum ExtraBehavior {
+    Allow,
+    Forbid,
+    Ignore,
+}
+
+impl ExtraBehavior {
+    pub fn from_schema_or_config(
+        py: Python,
+        schema: &PyDict,
+        config: Option<&PyDict>,
+        default: Self,
+    ) -> PyResult<Self> {
+        let extra_behavior = schema_or_config::<Option<&str>>(
+            schema,
+            config,
+            intern!(py, "extra_behavior"),
+            intern!(py, "extra_fields_behavior"),
+        )?
+        .flatten();
+        let res = match extra_behavior {
+            Some("allow") => Self::Allow,
+            Some("ignore") => Self::Ignore,
+            Some("forbid") => Self::Forbid,
+            Some(v) => return py_err!("Invalid extra_behavior: `{}`", v),
+            None => default,
+        };
+        Ok(res)
+    }
+}
