@@ -79,7 +79,7 @@ impl<'a> Input<'a> for PyAny {
     fn as_loc_item(&self) -> LocItem {
         if let Ok(py_str) = self.downcast::<PyString>() {
             py_str.to_string_lossy().as_ref().into()
-        } else if let Ok(key_int) = self.extract::<usize>() {
+        } else if let Ok(key_int) = self.extract::<i64>() {
             key_int.into()
         } else {
             safe_repr(self).to_string().into()
@@ -98,8 +98,8 @@ impl<'a> Input<'a> for PyAny {
         self.is_none()
     }
 
-    fn get_attr(&self, name: &PyString) -> Option<&PyAny> {
-        self.getattr(name).ok()
+    fn input_get_attr(&self, name: &PyString) -> Option<PyResult<&PyAny>> {
+        Some(self.getattr(name))
     }
 
     fn input_is_instance(&self, class: &PyAny, _json_mask: u8) -> PyResult<bool> {
@@ -110,8 +110,12 @@ impl<'a> Input<'a> for PyAny {
         Ok(result == 1)
     }
 
-    fn is_exact_instance(&self, class: &PyType) -> PyResult<bool> {
-        self.get_type().eq(class)
+    fn is_exact_instance(&self, class: &PyType) -> bool {
+        self.get_type().is(class)
+    }
+
+    fn is_python(&self) -> bool {
+        true
     }
 
     fn input_is_subclass(&self, class: &PyType) -> PyResult<bool> {
@@ -121,14 +125,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn maybe_subclass_dict(&self, class: &PyType) -> PyResult<&Self> {
-        if matches!(self.is_instance(class), Ok(true)) {
-            self.getattr(intern!(self.py(), "__dict__"))
-        } else {
-            Ok(self)
-        }
-    }
-
     fn input_as_url(&self) -> Option<PyUrl> {
         self.extract::<PyUrl>().ok()
     }
