@@ -8,13 +8,62 @@ use pyo3::{intern, AsPyPointer};
 use ahash::AHashSet;
 use serde::ser::Error;
 
-use crate::build_tools::py_err;
-
 use super::config::SerializationConfig;
-use super::errors::{PydanticSerializationUnexpectedValue, UNEXPECTED_TYPE_SER};
+use super::errors::{PydanticSerializationUnexpectedValue, UNEXPECTED_TYPE_SER_MARKER};
 use super::ob_type::ObTypeLookup;
 use super::shared::CombinedSerializer;
 
+/// this is ugly, would be much better if extra could be stored in `SerializationState`
+/// then `SerializationState` got a `serialize_infer` method, but I couldn't get it to work
+pub(crate) struct SerializationState {
+    warnings: CollectWarnings,
+    rec_guard: SerRecursionGuard,
+    config: SerializationConfig,
+}
+
+impl SerializationState {
+    pub fn new(timedelta_mode: Option<&str>, bytes_mode: Option<&str>) -> Self {
+        let warnings = CollectWarnings::new(None);
+        let rec_guard = SerRecursionGuard::default();
+        let config = SerializationConfig::from_args(timedelta_mode, bytes_mode).unwrap();
+        Self {
+            warnings,
+            rec_guard,
+            config,
+        }
+    }
+
+    pub fn extra<'py>(
+        &'py self,
+        py: Python<'py>,
+        mode: &'py SerMode,
+        exclude_none: Option<bool>,
+        round_trip: Option<bool>,
+        serialize_unknown: Option<bool>,
+        fallback: Option<&'py PyAny>,
+    ) -> Extra<'py> {
+        Extra::new(
+            py,
+            mode,
+            &[],
+            None,
+            &self.warnings,
+            None,
+            None,
+            exclude_none,
+            round_trip,
+            &self.config,
+            &self.rec_guard,
+            serialize_unknown,
+            fallback,
+        )
+    }
+
+    pub fn final_check(&self, py: Python) -> PyResult<()> {
+        self.warnings.final_check(py)
+    }
+}
+
 /// Useful things which are passed around by type_serializers
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
@@ -37,6 +86,8 @@ pub(crate) struct Extra<'a> {
     // it will be None otherwise
     pub model: Option<&'a PyAny>,
     pub field_name: Option<&'a str>,
+    pub serialize_unknown: bool,
+    pub fallback: Option<&'a PyAny>,
 }
 
 impl<'a> Extra<'a> {
@@ -53,6 +104,8 @@ impl<'a> Extra<'a> {
         round_trip: Option<bool>,
         config: &'a SerializationConfig,
         rec_guard: &'a SerRecursionGuard,
+        serialize_unknown: Option<bool>,
+        fallback: Option<&'a PyAny>,
     ) -> Self {
         Self {
             mode,
@@ -69,8 +122,14 @@ impl<'a> Extra<'a> {
             check: SerCheck::None,
             model: None,
             field_name: None,
+            serialize_unknown: serialize_unknown.unwrap_or(false),
+            fallback,
         }
     }
+
+    pub fn serialize_infer<'py>(&'py self, value: &'py PyAny) -> super::infer::SerializeInfer<'py> {
+        super::infer::SerializeInfer::new(value, None, None, self)
+    }
 }
 
 #[derive(Clone, Copy, PartialEq, Eq)]
@@ -104,8 +163,10 @@ pub(crate) struct ExtraOwned {
     config: SerializationConfig,
     rec_guard: SerRecursionGuard,
     check: SerCheck,
-    model: Option<Py<PyAny>>,
+    model: Option<PyObject>,
     field_name: Option<String>,
+    serialize_unknown: bool,
+    fallback: Option<PyObject>,
 }
 
 impl ExtraOwned {
@@ -124,6 +185,8 @@ impl ExtraOwned {
             check: extra.check,
             model: extra.model.map(|v| v.into()),
             field_name: extra.field_name.map(|v| v.to_string()),
+            serialize_unknown: extra.serialize_unknown,
+            fallback: extra.fallback.map(|v| v.into()),
         }
     }
 
@@ -143,6 +206,8 @@ impl ExtraOwned {
             check: self.check,
             model: self.model.as_ref().map(|m| m.as_ref(py)),
             field_name: self.field_name.as_ref().map(|n| n.as_ref()),
+            serialize_unknown: self.serialize_unknown,
+            fallback: self.fallback.as_ref().map(|m| m.as_ref(py)),
         }
     }
 }
@@ -238,7 +303,7 @@ impl CollectWarnings {
             // note: I think this should never actually happen since we use `to_python(..., mode='json')` during
             // JSON serialisation to "try" union branches, but it's here for completeness/correctness
             // in particular, in future we could allow errors instead of warnings on fallback
-            Err(S::Error::custom(UNEXPECTED_TYPE_SER))
+            Err(S::Error::custom(UNEXPECTED_TYPE_SER_MARKER))
         } else {
             self.fallback_warning(field_type, value);
             Ok(())
@@ -305,9 +370,9 @@ impl SerRecursionGuard {
         let id = value.as_ptr() as usize;
         let mut info = self.info.borrow_mut();
         if !info.ids.insert(id) {
-            py_err!(PyValueError; "Circular reference detected (id repeated)")
+            Err(PyValueError::new_err("Circular reference detected (id repeated)"))
         } else if info.depth > Self::MAX_DEPTH {
-            py_err!(PyValueError; "Circular reference detected (depth exceeded)")
+            Err(PyValueError::new_err("Circular reference detected (depth exceeded)"))
         } else {
             info.depth += 1;
             Ok(id)
