@@ -1,11 +1,11 @@
-use pyo3::exceptions::{PyKeyError, PyTypeError};
+use pyo3::exceptions::PyKeyError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList, PyString, PyTuple, PyType};
 
 use ahash::AHashSet;
 
-use crate::build_tools::{is_strict, py_err, safe_repr, schema_or_config_same, SchemaDict};
+use crate::build_tools::{is_strict, py_err, schema_or_config_same, ExtraBehavior, SchemaDict};
 use crate::errors::{ErrorType, ValError, ValLineError, ValResult};
 use crate::input::{GenericArguments, Input};
 use crate::lookup_key::LookupKey;
@@ -13,7 +13,7 @@ use crate::recursion_guard::RecursionGuard;
 use crate::validators::function::convert_err;
 
 use super::arguments::{json_get, json_slice, py_get, py_slice};
-use super::model::{create_class, force_setattr};
+use super::model::{create_class, force_setattr, Revalidate};
 use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
 
 #[derive(Debug, Clone)]
@@ -24,6 +24,7 @@ struct Field {
     init_only: bool,
     lookup_key: LookupKey,
     validator: CombinedValidator,
+    frozen: bool,
 }
 
 #[derive(Debug, Clone)]
@@ -33,6 +34,8 @@ pub struct DataclassArgsValidator {
     init_only_count: Option<usize>,
     dataclass_name: String,
     validator_name: String,
+    extra_behavior: ExtraBehavior,
+    loc_by_alias: bool,
 }
 
 impl BuildValidator for DataclassArgsValidator {
@@ -47,6 +50,8 @@ impl BuildValidator for DataclassArgsValidator {
 
         let populate_by_name = schema_or_config_same(schema, config, intern!(py, "populate_by_name"))?.unwrap_or(false);
 
+        let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
+
         let fields_schema: &PyList = schema.get_as_req(intern!(py, "fields"))?;
         let mut fields: Vec<Field> = Vec::with_capacity(fields_schema.len());
 
@@ -91,6 +96,7 @@ impl BuildValidator for DataclassArgsValidator {
                 lookup_key,
                 validator,
                 init_only: field.get_as(intern!(py, "init_only"))?.unwrap_or(false),
+                frozen: field.get_as::<bool>(intern!(py, "frozen"))?.unwrap_or(false),
             });
         }
 
@@ -108,6 +114,8 @@ impl BuildValidator for DataclassArgsValidator {
             init_only_count,
             dataclass_name,
             validator_name,
+            extra_behavior,
+            loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
         }
         .into())
     }
@@ -165,15 +173,15 @@ impl Validator for DataclassArgsValidator {
 
                     let mut kw_value = None;
                     if let Some(kwargs) = $args.kwargs {
-                        if let Some((key, value)) = field.lookup_key.$get_method(kwargs)? {
-                            used_keys.insert(key);
-                            kw_value = Some(value);
+                        if let Some((lookup_path, value)) = field.lookup_key.$get_method(kwargs)? {
+                            used_keys.insert(lookup_path.first_key());
+                            kw_value = Some((lookup_path, value));
                         }
                     }
 
                     match (pos_value, kw_value) {
                         // found both positional and keyword arguments, error
-                        (Some(_), Some(kw_value)) => {
+                        (Some(_), Some((_, kw_value))) => {
                             errors.push(ValLineError::new_with_loc(
                                 ErrorType::MultipleArgumentValues,
                                 kw_value,
@@ -198,7 +206,7 @@ impl Validator for DataclassArgsValidator {
                             }
                         }
                         // found a keyword argument, validate it
-                        (None, Some(kw_value)) => {
+                        (None, Some((lookup_path, kw_value))) => {
                             match field
                                 .validator
                                 .validate(py, kw_value, &extra, slots, recursion_guard)
@@ -206,9 +214,9 @@ impl Validator for DataclassArgsValidator {
                                 Ok(value) => set_item!(field, value),
                                 Err(ValError::LineErrors(line_errors)) => {
                                     errors.extend(
-                                        line_errors
-                                            .into_iter()
-                                            .map(|err| err.with_outer_location(field.name.clone().into())),
+                                        line_errors.into_iter().map(|err| {
+                                            lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name)
+                                        }),
                                     );
                                 }
                                 Err(err) => return Err(err),
@@ -225,10 +233,11 @@ impl Validator for DataclassArgsValidator {
                             )? {
                                 set_item!(field, value);
                             } else {
-                                errors.push(ValLineError::new_with_loc(
+                                errors.push(field.lookup_key.error(
                                     ErrorType::MissingKeywordArgument,
                                     input,
-                                    field.name.clone(),
+                                    self.loc_by_alias,
+                                    &field.name,
                                 ));
                             }
                         }
@@ -254,11 +263,20 @@ impl Validator for DataclassArgsValidator {
                             match raw_key.strict_str() {
                                 Ok(either_str) => {
                                     if !used_keys.contains(either_str.as_cow()?.as_ref()) {
-                                        errors.push(ValLineError::new_with_loc(
-                                            ErrorType::UnexpectedKeywordArgument,
-                                            value,
-                                            raw_key.as_loc_item(),
-                                        ));
+                                        // Unknown / extra field
+                                        match self.extra_behavior {
+                                            ExtraBehavior::Forbid => {
+                                                errors.push(ValLineError::new_with_loc(
+                                                    ErrorType::UnexpectedKeywordArgument,
+                                                    value,
+                                                    raw_key.as_loc_item(),
+                                                ));
+                                            }
+                                            ExtraBehavior::Ignore => {}
+                                            ExtraBehavior::Allow => {
+                                                output_dict.set_item(either_str.as_py_string(py), value)?
+                                            }
+                                        }
                                     }
                                 }
                                 Err(ValError::LineErrors(line_errors)) => {
@@ -291,10 +309,6 @@ impl Validator for DataclassArgsValidator {
         }
     }
 
-    fn get_name(&self) -> &str {
-        &self.validator_name
-    }
-
     fn validate_assignment<'s, 'data: 's>(
         &'s self,
         py: Python<'data>,
@@ -307,7 +321,19 @@ impl Validator for DataclassArgsValidator {
     ) -> ValResult<'data, PyObject> {
         let dict: &PyDict = obj.downcast()?;
 
+        let ok = |output: PyObject| {
+            dict.set_item(field_name, output)?;
+            Ok(dict.to_object(py))
+        };
+
         if let Some(field) = self.fields.iter().find(|f| f.name == field_name) {
+            if field.frozen {
+                return Err(ValError::new_with_loc(
+                    ErrorType::FrozenField,
+                    field_value,
+                    field.name.to_string(),
+                ));
+            }
             // by using dict but removing the field in question, we match V1 behaviour
             let data_dict = dict.copy()?;
             if let Err(err) = data_dict.del_item(field_name) {
@@ -325,10 +351,7 @@ impl Validator for DataclassArgsValidator {
                 .validator
                 .validate(py, field_value, &next_extra, slots, recursion_guard)
             {
-                Ok(output) => {
-                    dict.set_item(field_name, output)?;
-                    Ok(dict.to_object(py))
-                }
+                Ok(output) => ok(output),
                 Err(ValError::LineErrors(line_errors)) => {
                     let errors = line_errors
                         .into_iter()
@@ -339,15 +362,33 @@ impl Validator for DataclassArgsValidator {
                 Err(err) => Err(err),
             }
         } else {
-            Err(ValError::new_with_loc(
-                ErrorType::NoSuchAttribute {
-                    attribute: field_name.to_string(),
-                },
-                field_value,
-                field_name.to_string(),
-            ))
+            // Handle extra (unknown) field
+            // We partially use the extra_behavior for initialization / validation
+            // to determine how to handle assignment
+            match self.extra_behavior {
+                // For dataclasses we allow assigning unknown fields
+                // to match stdlib dataclass behavior
+                ExtraBehavior::Allow => ok(field_value.to_object(py)),
+                _ => Err(ValError::new_with_loc(
+                    ErrorType::NoSuchAttribute {
+                        attribute: field_name.to_string(),
+                    },
+                    field_value,
+                    field_name.to_string(),
+                )),
+            }
         }
     }
+
+    fn get_name(&self) -> &str {
+        &self.validator_name
+    }
+
+    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
+        self.fields
+            .iter_mut()
+            .try_for_each(|field| field.validator.complete(build_context))
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -356,7 +397,9 @@ pub struct DataclassValidator {
     validator: Box<CombinedValidator>,
     class: Py<PyType>,
     post_init: Option<Py<PyString>>,
+    revalidate: Revalidate,
     name: String,
+    frozen: bool,
 }
 
 impl BuildValidator for DataclassValidator {
@@ -384,9 +427,15 @@ impl BuildValidator for DataclassValidator {
             validator: Box::new(validator),
             class: class.into(),
             post_init,
+            revalidate: Revalidate::from_str(schema_or_config_same(
+                schema,
+                config,
+                intern!(py, "revalidate_instances"),
+            )?)?,
             // as with model, get the class's `__name__`, not using `class.name()` since it uses `__qualname__`
             // which is not what we want here
             name: class.getattr(intern!(py, "__name__"))?.extract()?,
+            frozen: schema.get_as(intern!(py, "frozen"))?.unwrap_or(false),
         }
         .into())
     }
@@ -405,14 +454,20 @@ impl Validator for DataclassValidator {
             // in the case that self_instance is Some, we're calling validation from within `BaseModel.__init__`
             return self.validate_init(py, self_instance, input, extra, slots, recursion_guard);
         }
-        let class = self.class.as_ref(py);
 
-        // we only do the is_exact_instance in strict mode
-        // we run validation even if input is an exact class to cover the case where a vanilla dataclass has been
-        // created with invalid types
-        // in theory we could have a flag to skip validation for an exact type in some scenarios, but I'm not sure
-        // that's a good idea
-        if extra.strict.unwrap_or(self.strict) && !input.is_exact_instance(class)? {
+        // same logic as on models
+        let class = self.class.as_ref(py);
+        if input.input_is_instance(class, 0)? {
+            if self.revalidate.should_revalidate(input, class) {
+                let input = input.input_get_attr(intern!(py, "__dict__")).unwrap()?;
+                let val_output = self.validator.validate(py, input, extra, slots, recursion_guard)?;
+                let dc = create_class(self.class.as_ref(py))?;
+                self.set_dict_call(py, dc.as_ref(py), val_output, input)?;
+                Ok(dc)
+            } else {
+                Ok(input.to_object(py))
+            }
+        } else if extra.strict.unwrap_or(self.strict) && input.is_python() {
             Err(ValError::new(
                 ErrorType::ModelClassType {
                     class_name: self.get_name().to_string(),
@@ -420,7 +475,6 @@ impl Validator for DataclassValidator {
                 input,
             ))
         } else {
-            let input = input.maybe_subclass_dict(class)?;
             let val_output = self.validator.validate(py, input, extra, slots, recursion_guard)?;
             let dc = create_class(self.class.as_ref(py))?;
             self.set_dict_call(py, dc.as_ref(py), val_output, input)?;
@@ -428,10 +482,6 @@ impl Validator for DataclassValidator {
         }
     }
 
-    fn get_name(&self) -> &str {
-        &self.name
-    }
-
     fn validate_assignment<'s, 'data: 's>(
         &'s self,
         py: Python<'data>,
@@ -442,11 +492,11 @@ impl Validator for DataclassValidator {
         slots: &'data [CombinedValidator],
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
-        let dict_attr = intern!(py, "__dict__");
-        let dict: &PyDict = match obj.get_attr(dict_attr) {
-            Some(v) => v.downcast()?,
-            None => return Err(PyTypeError::new_err(format!("{} is not a model instance", safe_repr(obj))).into()),
-        };
+        if self.frozen {
+            return Err(ValError::new(ErrorType::FrozenInstance, field_value));
+        }
+        let dict_py_str = intern!(py, "__dict__");
+        let dict: &PyDict = obj.getattr(dict_py_str)?.downcast()?;
 
         let new_dict = dict.copy()?;
         new_dict.set_item(field_name, field_value)?;
@@ -455,10 +505,14 @@ impl Validator for DataclassValidator {
             self.validator
                 .validate_assignment(py, new_dict, field_name, field_value, extra, slots, recursion_guard)?;
 
-        force_setattr(py, obj, dict_attr, dc_dict)?;
+        force_setattr(py, obj, dict_py_str, dc_dict)?;
 
         Ok(obj.to_object(py))
     }
+
+    fn get_name(&self) -> &str {
+        &self.name
+    }
 }
 
 impl DataclassValidator {
