@@ -25,7 +25,7 @@ macro_rules! set_build {
         fn build(
             schema: &PyDict,
             config: Option<&PyDict>,
-            build_context: &mut BuildContext,
+            build_context: &mut BuildContext<CombinedValidator>,
         ) -> PyResult<CombinedValidator> {
             let py = schema.py();
             let item_validator = get_items_schema(schema, config, build_context)?;
@@ -94,7 +94,7 @@ impl Validator for SetValidator {
         &self.name
     }
 
-    fn complete(&mut self, build_context: &BuildContext) -> PyResult<()> {
+    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
         match self.item_validator {
             Some(ref mut v) => v.complete(build_context),
             None => Ok(()),
