@@ -6,7 +6,7 @@ use pyo3::exceptions::PyKeyError;
 use pyo3::types::PyTuple;
 use pyo3::types::{PyDict, PySet, PyString, PyType};
 
-use crate::build_tools::{is_strict, py_err, schema_or_config, schema_or_config_same, SchemaDict};
+use crate::build_tools::{is_strict, py_err, schema_or_config, schema_or_config_same, ExtraBehavior, SchemaDict};
 use crate::errors::{py_err_string, ErrorType, ValError, ValLineError, ValResult};
 use crate::input::{
     AttributesGenericIterator, DictGenericIterator, GenericMapping, Input, JsonObjectGenericIterator,
@@ -31,12 +31,12 @@ struct TypedDictField {
 #[derive(Debug, Clone)]
 pub struct TypedDictValidator {
     fields: Vec<TypedDictField>,
-    check_extra: bool,
-    forbid_extra: bool,
+    extra_behavior: ExtraBehavior,
     extra_validator: Option<Box<CombinedValidator>>,
     strict: bool,
     from_attributes: bool,
     return_fields_set: bool,
+    loc_by_alias: bool,
 }
 
 impl BuildValidator for TypedDictValidator {
@@ -50,12 +50,6 @@ impl BuildValidator for TypedDictValidator {
         let py = schema.py();
         let strict = is_strict(schema, config)?;
 
-        let extra_behavior = schema_or_config::<&str>(
-            schema,
-            config,
-            intern!(py, "extra_behavior"),
-            intern!(py, "typed_dict_extra_behavior"),
-        )?;
         let total =
             schema_or_config(schema, config, intern!(py, "total"), intern!(py, "typed_dict_total"))?.unwrap_or(true);
         let from_attributes = schema_or_config_same(schema, config, intern!(py, "from_attributes"))?.unwrap_or(false);
@@ -63,25 +57,12 @@ impl BuildValidator for TypedDictValidator {
 
         let return_fields_set = schema.get_as(intern!(py, "return_fields_set"))?.unwrap_or(false);
 
-        let (check_extra, forbid_extra) = match extra_behavior {
-            Some(s) => match s {
-                "allow" => (true, false),
-                "ignore" => (false, false),
-                "forbid" => (true, true),
-                _ => return py_err!(r#"Invalid extra_behavior: "{}""#, s),
-            },
-            None => (false, false),
-        };
+        let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
 
-        let extra_validator = match schema.get_item(intern!(py, "extra_validator")) {
-            Some(v) => {
-                if check_extra && !forbid_extra {
-                    Some(Box::new(build_validator(v, config, build_context)?))
-                } else {
-                    return py_err!("extra_validator can only be used if extra_behavior=allow");
-                }
-            }
-            None => None,
+        let extra_validator = match (schema.get_item(intern!(py, "extra_validator")), &extra_behavior) {
+            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(v, config, build_context)?)),
+            (Some(_), _) => return py_err!("extra_validator can only be used if extra_behavior=allow"),
+            (_, _) => None,
         };
 
         let fields_dict: &PyDict = schema.get_as_req(intern!(py, "fields"))?;
@@ -143,12 +124,12 @@ impl BuildValidator for TypedDictValidator {
 
         Ok(Self {
             fields,
-            check_extra,
-            forbid_extra,
+            extra_behavior,
             extra_validator,
             strict,
             from_attributes,
             return_fields_set,
+            loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
         }
         .into())
     }
@@ -175,9 +156,9 @@ impl Validator for TypedDictValidator {
 
         // we only care about which keys have been used if we're iterating over the object for extra after
         // the first pass
-        let mut used_keys: Option<AHashSet<&str>> = match self.check_extra {
-            true => Some(AHashSet::with_capacity(self.fields.len())),
-            false => None,
+        let mut used_keys: Option<AHashSet<&str>> = match self.extra_behavior {
+            ExtraBehavior::Allow | ExtraBehavior::Forbid => Some(AHashSet::with_capacity(self.fields.len())),
+            _ => None,
         };
 
         macro_rules! process {
@@ -201,11 +182,11 @@ impl Validator for TypedDictValidator {
                             continue;
                         }
                     };
-                    if let Some((used_key, value)) = op_key_value {
+                    if let Some((lookup_path, value)) = op_key_value {
                         if let Some(ref mut used_keys) = used_keys {
                             // key is "used" whether or not validation passes, since we want to skip this key in
                             // extra logic either way
-                            used_keys.insert(used_key);
+                            used_keys.insert(lookup_path.first_key());
                         }
                         match field
                             .validator
@@ -220,7 +201,7 @@ impl Validator for TypedDictValidator {
                             Err(ValError::Omit) => continue,
                             Err(ValError::LineErrors(line_errors)) => {
                                 for err in line_errors {
-                                    errors.push(err.with_outer_location(field.name.clone().into()));
+                                    errors.push(lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name));
                                 }
                             }
                             Err(err) => return Err(err),
@@ -229,10 +210,11 @@ impl Validator for TypedDictValidator {
                     } else if let Some(value) = field.validator.default_value(py, Some(field.name.as_str()), &extra, slots, recursion_guard)? {
                         output_dict.set_item(&field.name_py, value)?;
                     } else if field.required {
-                        errors.push(ValLineError::new_with_loc(
+                        errors.push(field.lookup_key.error(
                             ErrorType::Missing,
                             input,
-                            field.name.clone(),
+                            self.loc_by_alias,
+                            &field.name
                         ));
                     }
                 }
@@ -257,39 +239,39 @@ impl Validator for TypedDictValidator {
                             continue;
                         }
 
-                        if self.forbid_extra {
-                            errors.push(ValLineError::new_with_loc(
-                                ErrorType::ExtraForbidden,
-                                value,
-                                raw_key.as_loc_item(),
-                            ));
-                            continue;
-                        }
-
-                        let py_key = either_str.as_py_string(py);
-                        if let Some(ref mut fs) = fields_set_vec {
-                            fs.push(py_key.into_py(py));
-                        }
-
-                        if let Some(ref validator) = self.extra_validator {
-                            match validator.validate(py, value, &extra, slots, recursion_guard) {
-                                Ok(value) => {
-                                    output_dict.set_item(py_key, value)?;
+                        // Unknown / extra field
+                        match self.extra_behavior {
+                            ExtraBehavior::Forbid => {
+                                errors.push(ValLineError::new_with_loc(
+                                    ErrorType::ExtraForbidden,
+                                    value,
+                                    raw_key.as_loc_item(),
+                                ));
+                            }
+                            ExtraBehavior::Ignore => {}
+                            ExtraBehavior::Allow => {
+                            let py_key = either_str.as_py_string(py);
+                                if let Some(ref validator) = self.extra_validator {
+                                    match validator.validate(py, value, &extra, slots, recursion_guard) {
+                                        Ok(value) => {
+                                            output_dict.set_item(py_key, value)?;
+                                            if let Some(ref mut fs) = fields_set_vec {
+                                                fs.push(py_key.into_py(py));
+                                            }
+                                        }
+                                        Err(ValError::LineErrors(line_errors)) => {
+                                            for err in line_errors {
+                                                errors.push(err.with_outer_location(raw_key.as_loc_item()));
+                                            }
+                                        }
+                                        Err(err) => return Err(err),
+                                    }
+                                } else {
+                                    output_dict.set_item(py_key, value.to_object(py))?;
                                     if let Some(ref mut fs) = fields_set_vec {
                                         fs.push(py_key.into_py(py));
                                     }
-                                }
-                                Err(ValError::LineErrors(line_errors)) => {
-                                    for err in line_errors {
-                                        errors.push(err.with_outer_location(raw_key.as_loc_item()));
-                                    }
-                                }
-                                Err(err) => return Err(err),
-                            }
-                        } else {
-                            output_dict.set_item(py_key, value.to_object(py))?;
-                            if let Some(ref mut fs) = fields_set_vec {
-                                fs.push(py_key.into_py(py));
+                                };
                             }
                         }
                     }
@@ -313,22 +295,6 @@ impl Validator for TypedDictValidator {
         }
     }
 
-    fn get_name(&self) -> &str {
-        Self::EXPECTED_TYPE
-    }
-
-    fn ask(&self, question: &Question) -> bool {
-        match question {
-            Question::ReturnFieldsSet => self.return_fields_set,
-        }
-    }
-
-    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
-        self.fields
-            .iter_mut()
-            .try_for_each(|f| f.validator.complete(build_context))
-    }
-
     fn validate_assignment<'s, 'data: 's>(
         &'s self,
         py: Python<'data>,
@@ -387,25 +353,29 @@ impl Validator for TypedDictValidator {
                         .validate(py, field_value, &extra, slots, recursion_guard),
                 )
             }
-        } else if self.check_extra && !self.forbid_extra {
-            // this is the "allow" case of extra_behavior
-            match self.extra_validator {
-                Some(ref validator) => {
-                    prepare_result(validator.validate(py, field_value, &extra, slots, recursion_guard))
-                }
-                None => ok(field_value.to_object(py)),
-            }
         } else {
-            // otherwise we raise an error:
-            // - with forbid this is obvious
-            // - with ignore the model should never be overloaded, so an error is the clearest option
-            Err(ValError::new_with_loc(
-                ErrorType::NoSuchAttribute {
-                    attribute: field_name.to_string(),
+            // Handle extra (unknown) field
+            // We partially use the extra_behavior for initialization / validation
+            // to determine how to handle assignment
+            // For models / typed dicts we forbid assigning extra attributes
+            // unless the user explicitly set extra_behavior to 'allow'
+            match self.extra_behavior {
+                ExtraBehavior::Allow => match self.extra_validator {
+                    Some(ref validator) => {
+                        prepare_result(validator.validate(py, field_value, &extra, slots, recursion_guard))
+                    }
+                    None => ok(field_value.to_object(py)),
                 },
-                field_value,
-                field_name.to_string(),
-            ))
+                ExtraBehavior::Forbid | ExtraBehavior::Ignore => {
+                    return Err(ValError::new_with_loc(
+                        ErrorType::NoSuchAttribute {
+                            attribute: field_name.to_string(),
+                        },
+                        field_value,
+                        field_name.to_string(),
+                    ))
+                }
+            }
         }?;
         if self.return_fields_set {
             let fields_set: &PySet = PySet::new(py, &[field_name.to_string()])?;
@@ -414,4 +384,24 @@ impl Validator for TypedDictValidator {
             Ok(new_data)
         }
     }
+
+    fn get_name(&self) -> &str {
+        Self::EXPECTED_TYPE
+    }
+
+    fn ask(&self, question: &Question) -> bool {
+        match question {
+            Question::ReturnFieldsSet => self.return_fields_set,
+        }
+    }
+
+    fn complete(&mut self, build_context: &BuildContext<CombinedValidator>) -> PyResult<()> {
+        self.fields
+            .iter_mut()
+            .try_for_each(|f| f.validator.complete(build_context))?;
+        match &mut self.extra_validator {
+            Some(v) => v.complete(build_context),
+            None => Ok(()),
+        }
+    }
 }
