@@ -8,14 +8,14 @@ from pydantic_core import PydanticSerializationUnexpectedValue, SchemaSerializer
 
 @pytest.mark.parametrize('input_value,expected_value', [(True, True), (False, False), (1, 1), (123, 123), (-42, -42)])
 def test_union_bool_int(input_value, expected_value):
-    s = SchemaSerializer(core_schema.union_schema(core_schema.bool_schema(), core_schema.int_schema()))
+    s = SchemaSerializer(core_schema.union_schema([core_schema.bool_schema(), core_schema.int_schema()]))
     assert s.to_python(input_value) == expected_value
     assert s.to_python(input_value, mode='json') == expected_value
     assert s.to_json(input_value) == json.dumps(expected_value).encode()
 
 
 def test_union_error():
-    s = SchemaSerializer(core_schema.union_schema(core_schema.bool_schema(), core_schema.int_schema()))
+    s = SchemaSerializer(core_schema.union_schema([core_schema.bool_schema(), core_schema.int_schema()]))
     msg = 'Expected `Union[bool, int]` but got `str` - serialized value may not be as expected'
     with pytest.warns(UserWarning, match=re.escape(msg)):
         assert s.to_python('a string') == 'a string'
@@ -114,7 +114,10 @@ def test_keys():
     s = SchemaSerializer(
         core_schema.dict_schema(
             core_schema.union_schema(
-                core_schema.int_schema(), core_schema.float_schema(serialization=core_schema.format_ser_schema('0.0f'))
+                [
+                    core_schema.int_schema(),
+                    core_schema.float_schema(serialization=core_schema.format_ser_schema('0.0f')),
+                ]
             ),
             core_schema.int_schema(),
         )
@@ -132,10 +135,12 @@ def test_union_of_functions():
 
     s = SchemaSerializer(
         core_schema.union_schema(
-            core_schema.any_schema(
-                serialization=core_schema.general_plain_serializer_function_ser_schema(repr_function)
-            ),
-            core_schema.float_schema(serialization=core_schema.format_ser_schema('_^14')),
+            [
+                core_schema.any_schema(
+                    serialization=core_schema.general_plain_serializer_function_ser_schema(repr_function)
+                ),
+                core_schema.float_schema(serialization=core_schema.format_ser_schema('_^14')),
+            ]
         )
     )
     assert s.to_python('foobar') == "func: 'foobar'"
@@ -150,22 +155,24 @@ def test_union_of_functions():
 def test_typed_dict_literal():
     s = SchemaSerializer(
         core_schema.union_schema(
-            core_schema.typed_dict_schema(
-                dict(
-                    pet_type=core_schema.typed_dict_field(core_schema.literal_schema('cat')),
-                    sound=core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
-                    ),
-                )
-            ),
-            core_schema.typed_dict_schema(
-                dict(
-                    pet_type=core_schema.typed_dict_field(core_schema.literal_schema('dog')),
-                    sound=core_schema.typed_dict_field(
-                        core_schema.float_schema(serialization=core_schema.format_ser_schema('0.3f'))
-                    ),
-                )
-            ),
+            [
+                core_schema.typed_dict_schema(
+                    dict(
+                        pet_type=core_schema.typed_dict_field(core_schema.literal_schema(['cat'])),
+                        sound=core_schema.typed_dict_field(
+                            core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
+                        ),
+                    )
+                ),
+                core_schema.typed_dict_schema(
+                    dict(
+                        pet_type=core_schema.typed_dict_field(core_schema.literal_schema(['dog'])),
+                        sound=core_schema.typed_dict_field(
+                            core_schema.float_schema(serialization=core_schema.format_ser_schema('0.3f'))
+                        ),
+                    )
+                ),
+            ]
         )
     )
 
@@ -179,15 +186,17 @@ def test_typed_dict_missing():
     """
     s = SchemaSerializer(
         core_schema.union_schema(
-            core_schema.typed_dict_schema(dict(foo=core_schema.typed_dict_field(core_schema.int_schema()))),
-            core_schema.typed_dict_schema(
-                dict(
-                    foo=core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
-                    ),
-                    bar=core_schema.typed_dict_field(core_schema.int_schema()),
-                )
-            ),
+            [
+                core_schema.typed_dict_schema(dict(foo=core_schema.typed_dict_field(core_schema.int_schema()))),
+                core_schema.typed_dict_schema(
+                    dict(
+                        foo=core_schema.typed_dict_field(
+                            core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
+                        ),
+                        bar=core_schema.typed_dict_field(core_schema.int_schema()),
+                    )
+                ),
+            ]
         )
     )
 
@@ -205,19 +214,21 @@ def test_typed_dict_extra():
     """
     s = SchemaSerializer(
         core_schema.union_schema(
-            core_schema.typed_dict_schema(
-                dict(
-                    foo=core_schema.typed_dict_field(core_schema.int_schema()),
-                    bar=core_schema.typed_dict_field(core_schema.int_schema()),
-                )
-            ),
-            core_schema.typed_dict_schema(
-                dict(
-                    foo=core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
+            [
+                core_schema.typed_dict_schema(
+                    dict(
+                        foo=core_schema.typed_dict_field(core_schema.int_schema()),
+                        bar=core_schema.typed_dict_field(core_schema.int_schema()),
                     )
-                )
-            ),
+                ),
+                core_schema.typed_dict_schema(
+                    dict(
+                        foo=core_schema.typed_dict_field(
+                            core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
+                        )
+                    )
+                ),
+            ]
         )
     )
 
@@ -235,20 +246,22 @@ def test_typed_dict_different_fields():
     """
     s = SchemaSerializer(
         core_schema.union_schema(
-            core_schema.typed_dict_schema(
-                dict(
-                    foo=core_schema.typed_dict_field(core_schema.int_schema()),
-                    bar=core_schema.typed_dict_field(core_schema.int_schema()),
-                )
-            ),
-            core_schema.typed_dict_schema(
-                dict(
-                    spam=core_schema.typed_dict_field(core_schema.int_schema()),
-                    ham=core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
-                    ),
-                )
-            ),
+            [
+                core_schema.typed_dict_schema(
+                    dict(
+                        foo=core_schema.typed_dict_field(core_schema.int_schema()),
+                        bar=core_schema.typed_dict_field(core_schema.int_schema()),
+                    )
+                ),
+                core_schema.typed_dict_schema(
+                    dict(
+                        spam=core_schema.typed_dict_field(core_schema.int_schema()),
+                        ham=core_schema.typed_dict_field(
+                            core_schema.int_schema(serialization=core_schema.format_ser_schema('04d'))
+                        ),
+                    )
+                ),
+            ]
         )
     )
 
