@@ -1,7 +1,9 @@
+import json
+import re
 from decimal import Decimal
 
 import pytest
-from dirty_equals import IsInstance
+from dirty_equals import HasRepr, IsInstance, IsStr
 
 from pydantic_core import (
     PydanticCustomError,
@@ -186,7 +188,7 @@ all_errors = [
     ('invalid_key', 'Keys should be strings', None),
     ('get_attribute_error', 'Error extracting attribute: foo', {'error': 'foo'}),
     ('model_class_type', 'Input should be an instance of foo', {'class_name': 'foo'}),
-    ('none_required', 'Input should be None/null', None),
+    ('none_required', 'Input should be None', None),
     ('bool', 'Input should be a valid boolean', None),
     ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
     ('greater_than', 'Input should be greater than 42.1', {'gt': '42.1'}),
@@ -215,7 +217,7 @@ all_errors = [
     ('mapping_type', 'Input should be a valid mapping, error: foobar', {'error': 'foobar'}),
     ('iterable_type', 'Input should be iterable', None),
     ('iteration_error', 'Error iterating over object, error: foobar', {'error': 'foobar'}),
-    ('list_type', 'Input should be a valid list/array', None),
+    ('list_type', 'Input should be a valid list', None),
     ('tuple_type', 'Input should be a valid tuple', None),
     ('set_type', 'Input should be a valid set', None),
     ('bool_type', 'Input should be a valid boolean', None),
@@ -362,29 +364,41 @@ def test_all_errors():
     assert errors[:4] == [
         {
             'type': 'no_such_attribute',
-            'message_template': "Object has no attribute '{attribute}'",
-            'example_message': "Object has no attribute ''",
+            'message_template_python': "Object has no attribute '{attribute}'",
+            'example_message_python': "Object has no attribute ''",
             'example_context': {'attribute': ''},
         },
         {
             'type': 'json_invalid',
-            'message_template': 'Invalid JSON: {error}',
-            'example_message': 'Invalid JSON: ',
+            'message_template_python': 'Invalid JSON: {error}',
+            'example_message_python': 'Invalid JSON: ',
             'example_context': {'error': ''},
         },
         {
             'type': 'json_type',
-            'message_template': 'JSON input should be string, bytes or bytearray',
-            'example_message': 'JSON input should be string, bytes or bytearray',
+            'message_template_python': 'JSON input should be string, bytes or bytearray',
+            'example_message_python': 'JSON input should be string, bytes or bytearray',
             'example_context': None,
         },
         {
             'type': 'recursion_loop',
-            'message_template': 'Recursion error - cyclic reference detected',
-            'example_message': 'Recursion error - cyclic reference detected',
+            'message_template_python': 'Recursion error - cyclic reference detected',
+            'example_message_python': 'Recursion error - cyclic reference detected',
             'example_context': None,
         },
     ]
+
+    none_required = next(e for e in errors if e['type'] == 'none_required')
+    # insert_assert(none_required)
+    assert none_required == {
+        'type': 'none_required',
+        'message_template_python': 'Input should be None',
+        'example_message_python': 'Input should be None',
+        'message_template_json': 'Input should be null',
+        'example_message_json': 'Input should be null',
+        'example_context': None,
+    }
+
     error_types = [e['type'] for e in errors]
     if error_types != list(core_schema.ErrorType.__args__):
         literal = ''.join(f'\n    {e!r},' for e in error_types)
@@ -410,3 +424,168 @@ def test_error_on_repr():
     assert exc_info.value.errors() == [
         {'type': 'int_type', 'loc': (), 'msg': 'Input should be a valid integer', 'input': IsInstance(BadRepr)}
     ]
+    assert json.loads(exc_info.value.json()) == [
+        {
+            'type': 'int_type',
+            'loc': [],
+            'msg': 'Input should be a valid integer',
+            'input': '<Unserializable BadRepr object>',
+        }
+    ]
+
+
+def test_error_json():
+    s = SchemaValidator({'type': 'str', 'min_length': 3})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('12')
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'string_too_short',
+            'loc': (),
+            'msg': 'String should have at least 3 characters',
+            'input': '12',
+            'ctx': {'min_length': 3},
+        }
+    ]
+    assert json.loads(exc_info.value.json()) == [
+        {
+            'type': 'string_too_short',
+            'loc': [],
+            'msg': 'String should have at least 3 characters',
+            'input': '12',
+            'ctx': {'min_length': 3},
+        }
+    ]
+    assert json.loads(exc_info.value.json(include_context=False)) == [
+        {'type': 'string_too_short', 'loc': [], 'msg': 'String should have at least 3 characters', 'input': '12'}
+    ]
+    assert exc_info.value.json().startswith('[{"type":"string_too_short",')
+    assert exc_info.value.json(indent=2).startswith('[\n  {\n    "type": "string_too_short",')
+
+
+def test_error_json_cycle():
+    s = SchemaValidator({'type': 'str', 'min_length': 3})
+    cycle = []
+    cycle.append(cycle)
+    msg = '[type=string_type, input_value=[[...]], input_type=list]'
+    with pytest.raises(ValidationError, match=re.escape(msg)) as exc_info:
+        s.validate_python(cycle)
+
+    assert json.loads(exc_info.value.json()) == [
+        {'type': 'string_type', 'loc': [], 'msg': 'Input should be a valid string', 'input': ['...']}
+    ]
+
+
+class Foobar:
+    pass
+
+
+class CustomStr:
+    def __str__(self):
+        return 'custom str'
+
+
+def test_error_json_unknown():
+    s = SchemaValidator({'type': 'str'})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python(Foobar())
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'string_type',
+            'loc': (),
+            'msg': 'Input should be a valid string',
+            'input': HasRepr(IsStr(regex='<.+.test_errors.Foobar object at 0x[a-f0-9]{5,}>', regex_flags=re.I)),
+        }
+    ]
+    # insert_assert(json.loads(exc_info.value.json()))
+    assert json.loads(exc_info.value.json()) == [
+        {
+            'type': 'string_type',
+            'loc': [],
+            'msg': 'Input should be a valid string',
+            'input': IsStr(regex='<.+.test_errors.Foobar object at 0x[a-f0-9]{5,}>', regex_flags=re.I),
+        }
+    ]
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python(CustomStr())
+    # insert_assert(json.loads(exc_info.value.json()))
+    assert json.loads(exc_info.value.json()) == [
+        {'type': 'string_type', 'loc': [], 'msg': 'Input should be a valid string', 'input': 'custom str'}
+    ]
+
+
+def test_error_json_loc():
+    s = SchemaValidator(
+        core_schema.dict_schema(core_schema.str_schema(), core_schema.list_schema(core_schema.int_schema()))
+    )
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python({'a': [0, 1, 'x'], 'b': [0, 'y']})
+
+    # insert_assert(json.loads(exc_info.value.json()))
+    assert json.loads(exc_info.value.json()) == [
+        {
+            'type': 'int_parsing',
+            'loc': ['a', 2],
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'x',
+        },
+        {
+            'type': 'int_parsing',
+            'loc': ['b', 1],
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'y',
+        },
+    ]
+
+
+def test_raise_validation_error():
+    with pytest.raises(ValidationError, match='1 validation error for Foobar\n') as exc_info:
+        raise ValidationError('Foobar', [{'type': 'greater_than', 'loc': ('a', 2), 'input': 4, 'ctx': {'gt': 5}}])
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {'type': 'greater_than', 'loc': (2, 'a'), 'msg': 'Input should be greater than 5', 'input': 4, 'ctx': {'gt': 5}}
+    ]
+    with pytest.raises(TypeError, match='GreaterThan requires context: {gt: Number}'):
+        raise ValidationError('Foobar', [{'type': 'greater_than', 'loc': ('a', 2), 'input': 4}])
+
+
+def test_raise_validation_error_json():
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError('Foobar', [{'type': 'none_required', 'loc': [-42], 'input': 'x'}])
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {'type': 'none_required', 'loc': (-42,), 'msg': 'Input should be None', 'input': 'x'}
+    ]
+
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError('Foobar', [{'type': 'none_required', 'loc': (), 'input': 'x'}], 'json')
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {'type': 'none_required', 'loc': (), 'msg': 'Input should be null', 'input': 'x'}
+    ]
+
+
+def test_raise_validation_error_custom():
+    custom_error = PydanticCustomError(
+        'my_error', 'this is a custom error {missed} {foo} {bar}', {'foo': 'X', 'bar': 42}
+    )
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError('Foobar', [{'type': custom_error, 'input': 'x'}])
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'my_error',
+            'loc': (),
+            'msg': 'this is a custom error {missed} X 42',
+            'input': 'x',
+            'ctx': {'foo': 'X', 'bar': 42},
+        }
+    ]
