@@ -1,6 +1,4 @@
 import re
-import sys
-from pathlib import Path
 
 import pytest
 from typing_extensions import get_args
@@ -101,7 +99,10 @@ def test_validation_error_multiple():
             'schema': {
                 'type': 'typed-dict',
                 'return_fields_set': True,
-                'fields': {'x': {'schema': {'type': 'float'}}, 'y': {'schema': {'type': 'int'}}},
+                'fields': {
+                    'x': {'type': 'typed-dict-field', 'schema': {'type': 'float'}},
+                    'y': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
+                },
             },
         }
     )
@@ -135,14 +136,6 @@ def test_validation_error_multiple():
     )
 
 
-@pytest.mark.skipif(sys.platform == 'emscripten', reason='README.md is not mounted in wasm file system')
-def test_readme(import_execute):
-    this_dir = Path(__file__).parent
-    readme = (this_dir / '..' / 'README.md').read_text()
-    example_code = re.search(r'\n```py\n(.*?)\n```\n', readme, re.M | re.S).group(1)
-    import_execute(example_code)
-
-
 def test_core_schema_type_literal():
     def get_type_value(schema):
         type_ = schema.__annotations__['type']
