@@ -52,17 +52,17 @@ all_schema_functions = [
     (core_schema.time_schema, args(), {'type': 'time'}),
     (core_schema.datetime_schema, args(), {'type': 'datetime'}),
     (core_schema.timedelta_schema, args(), {'type': 'timedelta'}),
-    (core_schema.literal_schema, args('a', 'b'), {'type': 'literal', 'expected': ['a', 'b']}),
+    (core_schema.literal_schema, args(['a', 'b']), {'type': 'literal', 'expected': ['a', 'b']}),
     (core_schema.is_instance_schema, args(int), {'type': 'is-instance', 'cls': int}),
     (core_schema.callable_schema, args(), {'type': 'callable'}),
     (core_schema.list_schema, args(), {'type': 'list'}),
     (core_schema.list_schema, args({'type': 'int'}), {'type': 'list', 'items_schema': {'type': 'int'}}),
     (
         core_schema.tuple_positional_schema,
-        args({'type': 'int'}),
+        args([{'type': 'int'}]),
         {'type': 'tuple-positional', 'items_schema': [{'type': 'int'}]},
     ),
-    (core_schema.tuple_positional_schema, args(), {'type': 'tuple-positional', 'items_schema': []}),
+    (core_schema.tuple_positional_schema, args([]), {'type': 'tuple-positional', 'items_schema': []}),
     (
         core_schema.tuple_variable_schema,
         args({'type': 'int'}),
@@ -131,12 +131,12 @@ all_schema_functions = [
     (core_schema.nullable_schema, args({'type': 'int'}), {'type': 'nullable', 'schema': {'type': 'int'}}),
     (
         core_schema.union_schema,
-        args({'type': 'int'}, {'type': 'str'}),
+        args([{'type': 'int'}, {'type': 'str'}]),
         {'type': 'union', 'choices': [{'type': 'int'}, {'type': 'str'}]},
     ),
     (
         core_schema.union_schema,
-        args({'type': 'int'}, {'type': 'str'}, custom_error_type='foobar', custom_error_message='This is Foobar'),
+        args([{'type': 'int'}, {'type': 'str'}], custom_error_type='foobar', custom_error_message='This is Foobar'),
         {
             'type': 'union',
             'choices': [{'type': 'int'}, {'type': 'str'}],
@@ -151,7 +151,7 @@ all_schema_functions = [
     ),
     (
         core_schema.chain_schema,
-        args({'type': 'int'}, {'type': 'str'}),
+        args([{'type': 'int'}, {'type': 'str'}]),
         {'type': 'chain', 'steps': [{'type': 'int'}, {'type': 'str'}]},
     ),
     (
@@ -173,8 +173,10 @@ all_schema_functions = [
     (
         core_schema.arguments_schema,
         args(
-            core_schema.arguments_parameter('foo', {'type': 'int'}),
-            core_schema.arguments_parameter('bar', {'type': 'str'}),
+            [
+                core_schema.arguments_parameter('foo', {'type': 'int'}),
+                core_schema.arguments_parameter('bar', {'type': 'str'}),
+            ],
             serialization=core_schema.format_ser_schema('d'),
         ),
         {
@@ -188,7 +190,7 @@ all_schema_functions = [
     ),
     (
         core_schema.call_schema,
-        args(core_schema.arguments_schema(core_schema.arguments_parameter('foo', {'type': 'int'})), val_function),
+        args(core_schema.arguments_schema([core_schema.arguments_parameter('foo', {'type': 'int'})]), val_function),
         {
             'type': 'call',
             'function': val_function,
@@ -270,14 +272,14 @@ def test_all_schema_functions_used():
 
 
 def test_invalid_custom_error():
-    s = core_schema.union_schema({'type': 'int'}, {'type': 'str'}, custom_error_type='foobar')
+    s = core_schema.union_schema([{'type': 'int'}, {'type': 'str'}], custom_error_type='foobar')
     with pytest.raises(SchemaError, match=r"KeyError: 'custom_error_message'"):
         SchemaValidator(s)
 
 
 def test_invalid_custom_error_type():
     s = core_schema.union_schema(
-        {'type': 'int'}, {'type': 'str'}, custom_error_type='finite_number', custom_error_message='x'
+        [{'type': 'int'}, {'type': 'str'}], custom_error_type='finite_number', custom_error_message='x'
     )
     msg = "custom_error.message should not be provided if 'custom_error_type' matches a known error"
     with pytest.raises(SchemaError, match=msg):
