@@ -44,8 +44,14 @@ def test_typed_dict(py_and_json: PyAndJson):
         {
             'type': 'typed-dict',
             'fields': {
-                'f1': {'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f1}}},
-                'f2': {'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f2}}},
+                'f1': {
+                    'type': 'typed-dict-field',
+                    'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f1}},
+                },
+                'f2': {
+                    'type': 'typed-dict-field',
+                    'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f2}},
+                },
             },
         }
     )
@@ -112,8 +118,14 @@ def test_validate_assignment_with_context():
         {
             'type': 'typed-dict',
             'fields': {
-                'f1': {'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f1}}},
-                'f2': {'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f2}}},
+                'f1': {
+                    'type': 'typed-dict-field',
+                    'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f1}},
+                },
+                'f2': {
+                    'type': 'typed-dict-field',
+                    'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': f2}},
+                },
             },
         }
     )
