@@ -1,8 +1,9 @@
 import dataclasses
 import re
+from typing import Any, Dict, Union
 
 import pytest
-from dirty_equals import IsListOrTuple
+from dirty_equals import IsListOrTuple, IsStr
 
 from pydantic_core import ArgsKwargs, SchemaValidator, ValidationError, core_schema
 
@@ -159,12 +160,12 @@ def test_dataclass_args_init_only(py_and_json: PyAndJson, input_value, expected)
         (
             ('hello',),
             Err(
-                'Input should be a dictionary or an instance of MyDataclass',
+                'Input should be (an object|a dictionary or an instance of MyDataclass)',
                 errors=[
                     {
                         'type': 'dataclass_type',
                         'loc': (),
-                        'msg': 'Input should be a dictionary or an instance of MyDataclass',
+                        'msg': IsStr(regex='Input should be (an object|a dictionary or an instance of MyDataclass)'),
                         'input': IsListOrTuple('hello'),
                         'ctx': {'dataclass_name': 'MyDataclass'},
                     }
@@ -180,7 +181,7 @@ def test_dataclass_args_init_only_no_fields(py_and_json: PyAndJson, input_value,
     v = py_and_json(schema)
 
     if isinstance(expected, Err):
-        with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
+        with pytest.raises(ValidationError, match=expected.message) as exc_info:
             v.validate_test(input_value)
 
         # debug(exc_info.value.errors())
@@ -267,16 +268,31 @@ class DuplicateDifferent:
 
 
 @pytest.mark.parametrize(
-    'input_value,expected',
+    'revalidate_instances,input_value,expected',
     [
-        ({'a': 'hello', 'b': True}, {'a': 'hello', 'b': True}),
-        (FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
-        (FooDataclassSame(a='hello', b=True), {'a': 'hello', 'b': True}),
-        (FooDataclassMore(a='hello', b=True, c='more'), Err(r'c\s+Unexpected keyword argument')),
-        (DuplicateDifferent(a='hello', b=True), Err('Input should be a dictionary or an instance of FooDataclass')),
+        ('always', {'a': 'hello', 'b': True}, {'a': 'hello', 'b': True}),
+        ('always', FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('always', FooDataclassSame(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('always', FooDataclassMore(a='hello', b=True, c='more'), Err(r'c\s+Unexpected keyword argument')),
+        ('always', DuplicateDifferent(a='hello', b=True), Err('should be a dictionary or an instance of FooDataclass')),
+        # revalidate_instances='subclass-instances'
+        ('subclass-instances', {'a': 'hello', 'b': True}, {'a': 'hello', 'b': True}),
+        ('subclass-instances', FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('subclass-instances', FooDataclass(a=b'hello', b='true'), {'a': b'hello', 'b': 'true'}),
+        ('subclass-instances', FooDataclassSame(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('subclass-instances', FooDataclassSame(a=b'hello', b='true'), {'a': 'hello', 'b': True}),
+        ('subclass-instances', FooDataclassMore(a='hello', b=True, c='more'), Err('Unexpected keyword argument')),
+        ('subclass-instances', DuplicateDifferent(a='hello', b=True), Err('dictionary or an instance of FooDataclass')),
+        # revalidate_instances='never'
+        ('never', {'a': 'hello', 'b': True}, {'a': 'hello', 'b': True}),
+        ('never', FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('never', FooDataclassSame(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('never', FooDataclassMore(a='hello', b=True, c='more'), {'a': 'hello', 'b': True, 'c': 'more'}),
+        ('never', FooDataclassMore(a='hello', b='wrong', c='more'), {'a': 'hello', 'b': 'wrong', 'c': 'more'}),
+        ('never', DuplicateDifferent(a='hello', b=True), Err('should be a dictionary or an instance of FooDataclass')),
     ],
 )
-def test_dataclass_subclass(input_value, expected):
+def test_dataclass_subclass(revalidate_instances, input_value, expected):
     schema = core_schema.dataclass_schema(
         FooDataclass,
         core_schema.dataclass_args_schema(
@@ -285,13 +301,15 @@ def test_dataclass_subclass(input_value, expected):
                 core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
                 core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
             ],
+            extra_behavior='forbid',
         ),
+        revalidate_instances=revalidate_instances,
     )
     v = SchemaValidator(schema)
 
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message) as exc_info:
-            v.validate_python(input_value)
+            print(v.validate_python(input_value))
 
         # debug(exc_info.value.errors())
         if expected.errors is not None:
@@ -302,6 +320,57 @@ def test_dataclass_subclass(input_value, expected):
         assert dataclasses.asdict(dc) == expected
 
 
+def test_dataclass_subclass_strict_never_revalidate():
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            FooDataclass,
+            core_schema.dataclass_args_schema(
+                'FooDataclass',
+                [
+                    core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                    core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
+                ],
+            ),
+            revalidate_instances='never',
+            strict=True,
+        )
+    )
+
+    foo = FooDataclass(a='hello', b=True)
+    assert v.validate_python(foo) is foo
+    sub_foo = FooDataclassSame(a='hello', b=True)
+    assert v.validate_python(sub_foo) is sub_foo
+
+    # this fails but that's fine, in realty `ArgsKwargs` should only be used via validate_init
+    with pytest.raises(ValidationError, match='Input should be an instance of FooDataclass'):
+        v.validate_python(ArgsKwargs((), {'a': 'hello', 'b': True}))
+
+
+def test_dataclass_subclass_subclass_revalidate():
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            FooDataclass,
+            core_schema.dataclass_args_schema(
+                'FooDataclass',
+                [
+                    core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                    core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
+                ],
+            ),
+            revalidate_instances='subclass-instances',
+            strict=True,
+        )
+    )
+
+    foo = FooDataclass(a='hello', b=True)
+    assert v.validate_python(foo) is foo
+    sub_foo = FooDataclassSame(a='hello', b='True')
+    sub_foo2 = v.validate_python(sub_foo)
+    assert sub_foo2 is not sub_foo
+    assert type(sub_foo2) is FooDataclass
+    assert dataclasses.asdict(sub_foo2) == dict(a='hello', b=True)
+
+
 def test_dataclass_post_init():
     @dataclasses.dataclass
     class Foo:
@@ -398,14 +467,17 @@ def test_dataclass_post_init_args_multiple():
 
 
 @pytest.mark.parametrize(
-    'input_value,expected',
+    'revalidate_instances,input_value,expected',
     [
-        ({'a': b'hello', 'b': 'true'}, {'a': 'hello', 'b': True}),
-        (FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
-        (FooDataclass(a=b'hello', b='true'), {'a': 'hello', 'b': True}),
+        ('always', {'a': b'hello', 'b': 'true'}, {'a': 'hello', 'b': True}),
+        ('always', FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('always', FooDataclass(a=b'hello', b='true'), {'a': 'hello', 'b': True}),
+        ('never', {'a': b'hello', 'b': 'true'}, {'a': 'hello', 'b': True}),
+        ('never', FooDataclass(a='hello', b=True), {'a': 'hello', 'b': True}),
+        ('never', FooDataclass(a=b'hello', b='true'), {'a': b'hello', 'b': 'true'}),
     ],
 )
-def test_dataclass_exact_validation(input_value, expected):
+def test_dataclass_exact_validation(revalidate_instances, input_value, expected):
     schema = core_schema.dataclass_schema(
         FooDataclass,
         core_schema.dataclass_args_schema(
@@ -415,6 +487,7 @@ def test_dataclass_exact_validation(input_value, expected):
                 core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
             ],
         ),
+        revalidate_instances=revalidate_instances,
     )
 
     v = SchemaValidator(schema)
@@ -623,7 +696,7 @@ def test_dataclass_self_init_alias():
             'Foo',
             [
                 core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), validation_alias='aAlias'),
-                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), validation_alias='bAlias'),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), validation_alias=['bAlias', 0]),
             ],
         ),
     )
@@ -634,10 +707,63 @@ def test_dataclass_self_init_alias():
 
     Foo.__init__ = __init__
 
-    foo = Foo(aAlias=b'hello', bAlias='True')
+    foo = Foo(aAlias=b'hello', bAlias=['True'])
     assert dataclasses.is_dataclass(foo)
     assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
 
+    with pytest.raises(ValidationError) as exc_info:
+        Foo(aAlias=b'hello', bAlias=['wrong'])
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'bool_parsing',
+            'loc': ('bAlias', 0),
+            'msg': 'Input should be a valid boolean, unable to interpret input',
+            'input': 'wrong',
+        }
+    ]
+
+
+def test_dataclass_self_init_alias_field_name():
+    @dataclasses.dataclass(init=False)
+    class Foo:
+        a: str
+        b: bool
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), validation_alias='aAlias'),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), validation_alias=['bAlias', 0]),
+            ],
+        ),
+    )
+    v = SchemaValidator(schema, {'loc_by_alias': False})
+
+    def __init__(self, *args, **kwargs):
+        v.validate_python(ArgsKwargs(args, kwargs), self_instance=self)
+
+    Foo.__init__ = __init__
+
+    foo = Foo(aAlias=b'hello', bAlias=['True'])
+    assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
+
+    with pytest.raises(ValidationError) as exc_info:
+        Foo(aAlias=b'hello', bAlias=['wrong'])
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'bool_parsing',
+            'loc': ('b',),
+            'msg': 'Input should be a valid boolean, unable to interpret input',
+            'input': 'wrong',
+        }
+    ]
+
 
 def test_dataclass_self_init_post_init():
     calls = []
@@ -702,19 +828,20 @@ def test_dataclass_validate_assignment():
     ]
 
     with pytest.raises(ValidationError) as exc_info:
-        v.validate_assignment(foo, 'c', 123)
+        v.validate_assignment(foo, 'c', '123')
     assert exc_info.value.errors() == [
         {
             'type': 'no_such_attribute',
             'loc': ('c',),
             'msg': "Object has no attribute 'c'",
-            'input': 123,
+            'input': '123',
             'ctx': {'attribute': 'c'},
         }
     ]
+    assert not hasattr(foo, 'c')
 
     # wrong arguments
-    with pytest.raises(TypeError, match="'field_a' is not a model instance"):
+    with pytest.raises(AttributeError, match="'str' object has no attribute '__dict__'"):
         v.validate_assignment('field_a', 'c', 123)
 
 
@@ -760,3 +887,180 @@ def test_validate_assignment_function():
         "ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
         "ValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
     ]
+
+
+def test_frozen():
+    @dataclasses.dataclass
+    class MyModel:
+        f: str
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyModel,
+            core_schema.dataclass_args_schema('MyModel', [core_schema.dataclass_field('f', core_schema.str_schema())]),
+            frozen=True,
+        )
+    )
+
+    m = v.validate_python({'f': 'x'})
+    assert m.f == 'x'
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'f', 'y')
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {'type': 'frozen_instance', 'loc': (), 'msg': 'Instance is frozen', 'input': 'y'}
+    ]
+
+
+def test_frozen_field():
+    @dataclasses.dataclass
+    class MyModel:
+        f: str
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyModel,
+            core_schema.dataclass_args_schema(
+                'MyModel', [core_schema.dataclass_field('f', core_schema.str_schema(), frozen=True)]
+            ),
+        )
+    )
+
+    m = v.validate_python({'f': 'x'})
+    assert m.f == 'x'
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'f', 'y')
+
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [{'type': 'frozen_field', 'loc': ('f',), 'msg': 'Field is frozen', 'input': 'y'}]
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='ignore'), {}),
+        (core_schema.CoreConfig(extra_fields_behavior='ignore'), {'extra_behavior': None}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'ignore'}),
+        (None, {'extra_behavior': 'ignore'}),
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': 'ignore'}),
+    ],
+)
+def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+    @dataclasses.dataclass
+    class MyModel:
+        f: str
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyModel,
+            core_schema.dataclass_args_schema(
+                'MyModel', [core_schema.dataclass_field('f', core_schema.str_schema())], **schema_extra_behavior_kw
+            ),
+        ),
+        config=config,
+    )
+
+    m: MyModel = v.validate_python({'f': 'x', 'extra_field': 123})
+    assert m.f == 'x'
+    assert not hasattr(m, 'extra_field')
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m.f == 'y'
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'not_f', 'xyz')
+
+    assert exc_info.value.errors() == [
+        {
+            'type': 'no_such_attribute',
+            'loc': ('not_f',),
+            'msg': "Object has no attribute 'not_f'",
+            'input': 'xyz',
+            'ctx': {'attribute': 'not_f'},
+        }
+    ]
+    assert not hasattr(m, 'not_f')
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {}),
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': None}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'forbid'}),
+        (None, {'extra_behavior': 'forbid'}),
+        (core_schema.CoreConfig(extra_fields_behavior='ignore'), {'extra_behavior': 'forbid'}),
+    ],
+)
+def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+    @dataclasses.dataclass
+    class MyModel:
+        f: str
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyModel,
+            core_schema.dataclass_args_schema(
+                'MyModel', [core_schema.dataclass_field('f', core_schema.str_schema())], **schema_extra_behavior_kw
+            ),
+        ),
+        config=config,
+    )
+
+    m: MyModel = v.validate_python({'f': 'x'})
+    assert m.f == 'x'
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m.f == 'y'
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'not_f', 'xyz')
+    assert exc_info.value.errors() == [
+        {
+            'type': 'no_such_attribute',
+            'loc': ('not_f',),
+            'msg': "Object has no attribute 'not_f'",
+            'input': 'xyz',
+            'ctx': {'attribute': 'not_f'},
+        }
+    ]
+    assert not hasattr(m, 'not_f')
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {}),
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': None}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'allow'}),
+        (None, {'extra_behavior': 'allow'}),
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': 'allow'}),
+    ],
+)
+def test_extra_behavior_allow(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+    @dataclasses.dataclass
+    class MyModel:
+        f: str
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyModel,
+            core_schema.dataclass_args_schema(
+                'MyModel', [core_schema.dataclass_field('f', core_schema.str_schema())], **schema_extra_behavior_kw
+            ),
+        ),
+        config=config,
+    )
+
+    m: MyModel = v.validate_python({'f': 'x', 'extra_field': '123'})
+    assert m.f == 'x'
+    assert getattr(m, 'extra_field') == '123'
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m.f == 'y'
+
+    v.validate_assignment(m, 'not_f', '123')
+    assert getattr(m, 'not_f') == '123'
