@@ -234,10 +234,43 @@ def test_function_wrap_invalid_location():
         {'type': 'function-wrap', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
     )
 
-    with pytest.raises(TypeError, match='^ValidatorCallable outer_location must be a str or int$'):
+    with pytest.raises(TypeError, match='^outer_location must be a str or int$'):
         v.validate_python(4)
 
 
+def test_function_after():
+    def f(input_value, _info):
+        return input_value + ' Changed'
+
+    v = SchemaValidator(
+        {'type': 'function-after', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'str'}}
+    )
+
+    assert v.validate_python('input value') == 'input value Changed'
+
+
+def test_function_after_raise():
+    def f(input_value, info):
+        raise ValueError('foobar')
+
+    v = SchemaValidator(
+        {'type': 'function-after', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'str'}}
+    )
+
+    with pytest.raises(ValidationError) as exc_info:
+        assert v.validate_python('input value') == 'input value Changed'
+    # debug(str(exc_info.value))
+    assert exc_info.value.errors() == [
+        {
+            'type': 'value_error',
+            'loc': (),
+            'msg': 'Value error, foobar',
+            'input': 'input value',
+            'ctx': {'error': 'foobar'},
+        }
+    ]
+
+
 def test_function_after_config():
     f_kwargs = None
 
@@ -733,12 +766,16 @@ def test_typed_dict_data() -> None:
     assert info_stuff == {'field_name': 'c', 'data': {'a': 1}}
 
 
-@pytest.mark.parametrize('mode', ['before', 'after', 'wrap'])
-def test_model_root_function_assignment(mode):
-    """
-    Root functional validators should not be called in validate assignment, since the input value makes no sense.
-    """
-    call_count = 0
+@pytest.mark.parametrize(
+    'mode,calls1,calls2',
+    [
+        ('before', {'value': {'x': b'input', 'y': '123'}}, {'value': {'x': 'different', 'y': 123}}),
+        ('after', {'value': {'x': 'input', 'y': 123}}, {'value': {'x': 'different', 'y': 123}}),
+        ('wrap', {'value': {'x': b'input', 'y': '123'}}, {'value': {'x': 'different', 'y': 123}}),
+    ],
+)
+def test_model_root_function_assignment(mode: str, calls1: Any, calls2: Any):
+    calls: list[Any] = []
 
     class Model:
         x: str
@@ -747,12 +784,13 @@ def test_model_root_function_assignment(mode):
         def __init__(self, **kwargs: Any) -> None:
             self.__dict__.update(kwargs)
 
-    def f(input_value: Any, *args) -> Any:
-        nonlocal call_count
-        call_count += 1
+    def f(input_value: Any, *args: Any) -> Any:
         if mode == 'wrap':
-            return args[0](input_value)
+            handler, _ = args
+            calls.append({'value': input_value})
+            return handler(input_value)
         else:
+            calls.append({'value': input_value})
             return input_value
 
     v = SchemaValidator(
@@ -775,7 +813,7 @@ def test_model_root_function_assignment(mode):
     v.validate_python({'x': b'input', 'y': '123'}, self_instance=m)
     assert m.x == 'input'
     assert m.y == 123
-    assert call_count == 1
+    assert calls == [calls1]
 
     v.validate_assignment(m, 'x', b'different')
-    assert call_count == 1
+    assert calls == [calls1, calls2]
