@@ -3,12 +3,12 @@ import re
 import sys
 from dataclasses import dataclass
 from datetime import datetime
-from typing import Any, Mapping
+from typing import Any, Dict, Mapping, Union
 
 import pytest
 from dirty_equals import FunctionCheck, HasRepr, IsStr
 
-from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError
+from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError, core_schema
 
 from ..conftest import Err, PyAndJson
 
@@ -125,12 +125,8 @@ field_b
         ({}, Map(a=123), {'a': 123}),
         ({}, {b'a': '123'}, Err('Field required [type=missing,')),
         ({}, {'a': '123', 'c': 4}, {'a': 123}),
-        ({'typed_dict_extra_behavior': 'allow'}, {'a': '123', 'c': 4}, {'a': 123, 'c': 4}),
-        (
-            {'typed_dict_extra_behavior': 'allow'},
-            {'a': '123', b'c': 4},
-            Err('Keys should be strings [type=invalid_key,'),
-        ),
+        ({'extra_fields_behavior': 'allow'}, {'a': '123', 'c': 4}, {'a': 123, 'c': 4}),
+        ({'extra_fields_behavior': 'allow'}, {'a': '123', b'c': 4}, Err('Keys should be strings [type=invalid_key,')),
         ({'strict': True}, Map(a=123), Err('Input should be a valid dictionary [type=dict_type,')),
         ({}, {'a': '123', 'b': '4.7'}, {'a': 123, 'b': 4.7}),
         ({}, {'a': '123', 'b': 'nan'}, {'a': 123, 'b': FunctionCheck(math.isnan)}),
@@ -198,50 +194,6 @@ def test_forbid_extra():
     ]
 
 
-def test_allow_extra():
-    v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'return_fields_set': True,
-            'fields': {'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-            'extra_behavior': 'allow',
-        }
-    )
-
-    assert v.validate_python({'field_a': b'abc', 'field_b': (1, 2)}) == (
-        {'field_a': 'abc', 'field_b': (1, 2)},
-        {'field_a', 'field_b'},
-    )
-
-
-def test_allow_extra_validate():
-    v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'return_fields_set': True,
-            'fields': {'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-            'extra_validator': {'type': 'int'},
-            'extra_behavior': 'allow',
-        }
-    )
-
-    assert v.validate_python({'field_a': 'test', 'other_value': '123'}) == (
-        {'field_a': 'test', 'other_value': 123},
-        {'field_a', 'other_value'},
-    )
-
-    with pytest.raises(ValidationError) as exc_info:
-        v.validate_python({'field_a': 'test', 'other_value': 12.5})
-    assert exc_info.value.errors() == [
-        {
-            'type': 'int_from_float',
-            'loc': ('other_value',),
-            'msg': 'Input should be a valid integer, got a number with a fractional part',
-            'input': 12.5,
-        }
-    ]
-
-
 def test_allow_extra_invalid():
     with pytest.raises(SchemaError, match='extra_validator can only be used if extra_behavior=allow'):
         SchemaValidator(
@@ -250,8 +202,8 @@ def test_allow_extra_invalid():
 
 
 def test_allow_extra_wrong():
-    with pytest.raises(SchemaError, match='Invalid extra_behavior: "wrong"'):
-        SchemaValidator({'type': 'typed-dict', 'fields': {}}, {'typed_dict_extra_behavior': 'wrong'})
+    with pytest.raises(SchemaError, match='Invalid extra_behavior: `wrong`'):
+        SchemaValidator({'type': 'typed-dict', 'fields': {}}, {'extra_fields_behavior': 'wrong'})
 
 
 def test_str_config():
@@ -600,9 +552,9 @@ def test_alias(py_and_json: PyAndJson):
         }
     )
     assert v.validate_test({'FieldA': '123'}) == {'field_a': 123}
-    with pytest.raises(ValidationError, match=r'field_a\n +Field required \[type=missing,'):
+    with pytest.raises(ValidationError, match=r'FieldA\n +Field required \[type=missing,'):
         assert v.validate_test({'foobar': '123'})
-    with pytest.raises(ValidationError, match=r'field_a\n +Field required \[type=missing,'):
+    with pytest.raises(ValidationError, match=r'FieldA\n +Field required \[type=missing,'):
         assert v.validate_test({'field_a': '123'})
 
 
@@ -645,7 +597,7 @@ def test_alias_allow_pop(py_and_json: PyAndJson):
     assert v.validate_test({'FieldA': '123'}) == ({'field_a': 123}, {'field_a'})
     assert v.validate_test({'field_a': '123'}) == ({'field_a': 123}, {'field_a'})
     assert v.validate_test({'FieldA': '1', 'field_a': '2'}) == ({'field_a': 1}, {'field_a'})
-    with pytest.raises(ValidationError, match=r'field_a\n +Field required \[type=missing,'):
+    with pytest.raises(ValidationError, match=r'FieldA\n +Field required \[type=missing,'):
         assert v.validate_test({'foobar': '123'})
 
 
@@ -653,10 +605,10 @@ def test_alias_allow_pop(py_and_json: PyAndJson):
     'input_value,expected',
     [
         ({'foo': {'bar': '123'}}, {'field_a': 123}),
-        ({'x': '123'}, Err(r'field_a\n +Field required \[type=missing,')),
-        ({'foo': '123'}, Err(r'field_a\n +Field required \[type=missing,')),
-        ({'foo': [1, 2, 3]}, Err(r'field_a\n +Field required \[type=missing,')),
-        ({'foo': {'bat': '123'}}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'x': '123'}, Err(r'foo -> bar\n +Field required \[type=missing,')),
+        ({'foo': '123'}, Err(r'foo -> bar\n +Field required \[type=missing,')),
+        ({'foo': [1, 2, 3]}, Err(r'foo -> bar\n +Field required \[type=missing,')),
+        ({'foo': {'bat': '123'}}, Err(r'foo -> bar\n +Field required \[type=missing,')),
     ],
     ids=repr,
 )
@@ -705,6 +657,69 @@ def test_aliases_path_multiple(py_and_json: PyAndJson, input_value, expected):
                     'schema': {'type': 'int'},
                 }
             },
+        },
+        {'loc_by_alias': False},
+    )
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=expected.message):
+            val = v.validate_test(input_value)
+            print(f'UNEXPECTED OUTPUT: {val!r}')
+    else:
+        output = v.validate_test(input_value)
+        assert output == expected
+
+
+@pytest.mark.parametrize(
+    'input_value,expected',
+    [
+        ({'foo': {-2: '123'}}, {'field_a': 123}),
+        # negatives indexes work fine
+        ({'foo': [1, 42, 'xx']}, {'field_a': 42}),
+        ({'foo': [42, 'xxx', 42]}, Err(r'Input should be a valid integer,')),
+        ({'foo': [42]}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'foo': {'xx': '123'}}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'foo': {'-2': '123'}}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'foo': {2: '123'}}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'foo': 'foobar'}, Err(r'field_a\n +Field required \[type=missing,')),
+        ({'foo': {0, 1, 2}}, Err(r'field_a\n +Field required \[type=missing,')),
+    ],
+    ids=repr,
+)
+def test_aliases_path_negative(input_value, expected):
+    v = SchemaValidator(
+        {
+            'type': 'typed-dict',
+            'fields': {
+                'field_a': {'validation_alias': ['foo', -2], 'type': 'typed-dict-field', 'schema': {'type': 'int'}}
+            },
+        },
+        {'loc_by_alias': False},
+    )
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=expected.message):
+            val = v.validate_python(input_value)
+            print(f'UNEXPECTED OUTPUT: {val!r}')
+    else:
+        output = v.validate_python(input_value)
+        assert output == expected
+
+
+@pytest.mark.parametrize(
+    'input_value,expected',
+    [
+        ({'foo': [1, 42, 'xx']}, {'field_a': 42}),
+        ({'foo': [42, 'xxx', 42]}, Err(r'Input should be a valid integer,')),
+        ({'foo': [42]}, Err(r'foo -> -2\n +Field required \[type=missing,')),
+    ],
+    ids=repr,
+)
+def test_aliases_path_negative_json(py_and_json: PyAndJson, input_value, expected):
+    v = py_and_json(
+        {
+            'type': 'typed-dict',
+            'fields': {
+                'field_a': {'validation_alias': ['foo', -2], 'type': 'typed-dict-field', 'schema': {'type': 'int'}}
+            },
         }
     )
     if isinstance(expected, Err):
@@ -807,23 +822,69 @@ def test_alias_build_error(alias_schema, error):
         )
 
 
-def test_alias_error_loc():
-    v = SchemaValidator(
+def test_alias_error_loc_alias(py_and_json: PyAndJson):
+    v = py_and_json(
         {
             'type': 'typed-dict',
             'fields': {
                 'field_a': {
                     'type': 'typed-dict-field',
                     'schema': {'type': 'int'},
-                    'validation_alias': [['bar'], ['foo']],
+                    'validation_alias': [['foo', 'x'], ['bar', 1, -1]],
                 }
             },
-        }
+        },
+        {'loc_by_alias': True},  # this is the default
     )
-    assert v.validate_python({'foo': 42}) == {'field_a': 42}
-    assert v.validate_python({'bar': 42}) == {'field_a': 42}
+    assert v.validate_test({'foo': {'x': 42}}) == {'field_a': 42}
+    assert v.validate_python({'bar': ['x', {-1: 42}]}) == {'field_a': 42}
+    assert v.validate_test({'bar': ['x', [1, 2, 42]]}) == {'field_a': 42}
     with pytest.raises(ValidationError) as exc_info:
-        v.validate_python({'foo': 'not_int'})
+        v.validate_test({'foo': {'x': 'not_int'}})
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'int_parsing',
+            'loc': ('foo', 'x'),
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'not_int',
+        }
+    ]
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_test({'bar': ['x', [1, 2, 'not_int']]})
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'type': 'int_parsing',
+            'loc': ('bar', 1, -1),
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'not_int',
+        }
+    ]
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_test({})
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [{'type': 'missing', 'loc': ('foo', 'x'), 'msg': 'Field required', 'input': {}}]
+
+
+def test_alias_error_loc_field_names(py_and_json: PyAndJson):
+    v = py_and_json(
+        {
+            'type': 'typed-dict',
+            'fields': {
+                'field_a': {
+                    'type': 'typed-dict-field',
+                    'schema': {'type': 'int'},
+                    'validation_alias': [['foo'], ['bar', 1, -1]],
+                }
+            },
+        },
+        {'loc_by_alias': False},
+    )
+    assert v.validate_test({'foo': 42}) == {'field_a': 42}
+    assert v.validate_test({'bar': ['x', [1, 2, 42]]}) == {'field_a': 42}
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_test({'foo': 'not_int'})
     # insert_assert(exc_info.value.errors())
     assert exc_info.value.errors() == [
         {
@@ -834,7 +895,7 @@ def test_alias_error_loc():
         }
     ]
     with pytest.raises(ValidationError) as exc_info:
-        v.validate_python({'bar': 'not_int'})
+        v.validate_test({'bar': ['x', [1, 2, 'not_int']]})
     # insert_assert(exc_info.value.errors())
     assert exc_info.value.errors() == [
         {
@@ -844,6 +905,10 @@ def test_alias_error_loc():
             'input': 'not_int',
         }
     ]
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_test({})
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [{'type': 'missing', 'loc': ('field_a',), 'msg': 'Field required', 'input': {}}]
 
 
 def test_empty_model():
@@ -1241,7 +1306,8 @@ def test_from_attributes_path(input_value, expected):
                 }
             },
             'from_attributes': True,
-        }
+        },
+        {'loc_by_alias': False},
     )
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=expected.message):
@@ -1297,7 +1363,8 @@ def test_alias_extra(py_and_json: PyAndJson):
                     'schema': {'type': 'int'},
                 }
             },
-        }
+        },
+        {'loc_by_alias': False},
     )
     assert v.validate_test({'FieldA': 1}) == {'field_a': 1}
     assert v.validate_test({'foo': [1, 2, 3]}) == {'field_a': 3}
@@ -1631,3 +1698,131 @@ def test_frozen_field():
     assert exc_info.value.errors() == [
         {'type': 'frozen_field', 'loc': ('is_developer',), 'msg': 'Field is frozen', 'input': False}
     ]
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {}),
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': None}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'allow'}),
+        (None, {'extra_behavior': 'allow'}),
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': 'allow'}),
+    ],
+)
+@pytest.mark.parametrize(
+    'extra_validator_kw, expected_extra_value',
+    [({}, '123'), ({'extra_validator': None}, '123'), ({'extra_validator': core_schema.int_schema()}, 123)],
+    ids=['extra_validator=unset', 'extra_validator=None', 'extra_validator=int'],
+)
+def test_extra_behavior_allow(
+    config: Union[core_schema.CoreConfig, None],
+    schema_extra_behavior_kw: Dict[str, Any],
+    extra_validator_kw: Dict[str, Any],
+    expected_extra_value: Any,
+):
+    v = SchemaValidator(
+        core_schema.typed_dict_schema(
+            {'f': core_schema.typed_dict_field(core_schema.str_schema())},
+            **schema_extra_behavior_kw,
+            **extra_validator_kw,
+        ),
+        config=config,
+    )
+
+    m: Dict[str, Any] = v.validate_python({'f': 'x', 'extra_field': '123'})
+    assert m == {'f': 'x', 'extra_field': expected_extra_value}
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m['f'] == 'y'
+
+    v.validate_assignment(m, 'not_f', '123')
+    assert m['not_f'] == expected_extra_value
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {}),
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': None}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'forbid'}),
+        (None, {'extra_behavior': 'forbid'}),
+        (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': 'forbid'}),
+    ],
+)
+def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+    v = SchemaValidator(
+        core_schema.typed_dict_schema(
+            {'f': core_schema.typed_dict_field(core_schema.str_schema())}, **schema_extra_behavior_kw
+        ),
+        config=config,
+    )
+
+    m: Dict[str, Any] = v.validate_python({'f': 'x'})
+    assert m == {'f': 'x'}
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python({'f': 'x', 'extra_field': 123})
+    assert exc_info.value.errors() == [
+        {'type': 'extra_forbidden', 'loc': ('extra_field',), 'msg': 'Extra inputs are not permitted', 'input': 123}
+    ]
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m['f'] == 'y'
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'not_f', 'xyz')
+    assert exc_info.value.errors() == [
+        {
+            'type': 'no_such_attribute',
+            'loc': ('not_f',),
+            'msg': "Object has no attribute 'not_f'",
+            'input': 'xyz',
+            'ctx': {'attribute': 'not_f'},
+        }
+    ]
+    assert 'not_f' not in m
+
+
+@pytest.mark.parametrize(
+    'config,schema_extra_behavior_kw',
+    [
+        (core_schema.CoreConfig(extra_fields_behavior='ignore'), {}),
+        (core_schema.CoreConfig(), {'extra_behavior': 'ignore'}),
+        (None, {'extra_behavior': 'ignore'}),
+        (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': 'ignore'}),
+        (core_schema.CoreConfig(), {}),
+        (core_schema.CoreConfig(), {'extra_behavior': None}),
+        (None, {'extra_behavior': None}),
+    ],
+)
+def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+    v = SchemaValidator(
+        core_schema.typed_dict_schema(
+            {'f': core_schema.typed_dict_field(core_schema.str_schema())}, **schema_extra_behavior_kw
+        ),
+        config=config,
+    )
+
+    m: Dict[str, Any] = v.validate_python({'f': 'x', 'extra_field': 123})
+    assert m == {'f': 'x'}
+
+    v.validate_assignment(m, 'f', 'y')
+    assert m['f'] == 'y'
+
+    # even if we ignore extra attributes during initialization / validation
+    # we never ignore them during assignment
+    # instead if extra='ignore' was set (or nothing was set since that's the default)
+    # we treat it as if it were extra='forbid'
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(m, 'not_f', 'xyz')
+    assert exc_info.value.errors() == [
+        {
+            'type': 'no_such_attribute',
+            'loc': ('not_f',),
+            'msg': "Object has no attribute 'not_f'",
+            'input': 'xyz',
+            'ctx': {'attribute': 'not_f'},
+        }
+    ]
+    assert 'not_f' not in m
